/* eslint-disable no-magic-numbers */
import { createAsyncThunk } from '@reduxjs/toolkit';
import { ZERO } from '@/constants/common';
import { QueryData } from '@/types/query';
import { DEFAULT_ZOOM } from '@/constants/map';
import { getPointMerged } from '@/utils/object/getPointMerged';
import type { AppDispatch, RootState } from '../store';
import {
  InitMapBackgroundActionPayload,
  InitMapBackgroundParams,
  InitMapPositionActionPayload,
  InitMapPositionParams,
  InitMapSizeAndModelIdActionPayload,
  InitMapSizeAndModelIdParams,
  InitOpenedMapsActionPayload,
  InitOpenedMapsProps,
  MapSizeAndModelId,
  OppenedMap,
  Position,
} from './map.types';
import { mainBackgroundsDataSelector } from '../backgrounds/background.selectors';
import {
  currentModelSelector,
  mainMapModelSelector,
  modelByIdSelector,
  modelsDataSelector,
} from '../models/models.selectors';
import { DEFAULT_POSITION, MAIN_MAP } from './map.constants';

/** UTILS - in the same file because of dependancy cycle */

export const getBackgroundId = (state: RootState, queryData: QueryData): number => {
  const mainMapBackground = mainBackgroundsDataSelector(state);
  const backgroundId = queryData?.backgroundId || mainMapBackground?.id || ZERO;

  return backgroundId;
};

export const getInitMapPosition = (state: RootState, queryData: QueryData): Position => {
  const mainMapModel = mainMapModelSelector(state);
  const modelId = queryData?.modelId || mainMapModel?.idObject || ZERO;
  const currentModel = modelByIdSelector(state, modelId);
  const position = queryData?.initialPosition;
  const HALF = 2;

  if (!currentModel) {
    return {
      last: DEFAULT_POSITION,
      initial: DEFAULT_POSITION,
    };
  }

  const defaultPosition = {
    x: currentModel.defaultCenterX ?? currentModel.width / HALF,
    y: currentModel.defaultCenterY ?? currentModel.height / HALF,
    z: currentModel.defaultZoomLevel ?? DEFAULT_ZOOM,
  };

  const mergedPosition = getPointMerged(position || {}, defaultPosition);

  return {
    last: mergedPosition,
    initial: mergedPosition,
  };
};

export const getInitMapSizeAndModelId = (
  state: RootState,
  queryData: QueryData,
): MapSizeAndModelId => {
  const mainMapModel = mainMapModelSelector(state);
  const modelId = queryData?.modelId || mainMapModel?.idObject || ZERO;
  const currentModel = modelByIdSelector(state, modelId);

  return {
    modelId: currentModel?.idObject || ZERO,
    size: {
      width: currentModel?.width || ZERO,
      height: currentModel?.height || ZERO,
      tileSize: currentModel?.tileSize || ZERO,
      minZoom: currentModel?.minZoom || ZERO,
      maxZoom: currentModel?.maxZoom || ZERO,
    },
  };
};

export const getOpenedMaps = (state: RootState, queryData: QueryData): OppenedMap[] => {
  const FIRST = 0;
  const models = modelsDataSelector(state);
  const currentModel = currentModelSelector(state);
  const mainMapId = models?.[FIRST]?.idObject || ZERO;

  const openedMaps: OppenedMap[] = [
    { modelId: mainMapId, modelName: MAIN_MAP, lastPosition: DEFAULT_POSITION },
  ];

  const isMainMapSetAsCurrentModel = currentModel?.idObject !== mainMapId;

  if (isMainMapSetAsCurrentModel) {
    openedMaps.push({
      modelId: currentModel?.idObject || ZERO,
      modelName: currentModel?.name || '',
      lastPosition: { ...DEFAULT_POSITION, ...queryData.initialPosition },
    });
  }
  return openedMaps;
};

/** THUNKS  */

export const initMapSizeAndModelId = createAsyncThunk<
  InitMapSizeAndModelIdActionPayload,
  InitMapSizeAndModelIdParams,
  { dispatch: AppDispatch; state: RootState }
>(
  'map/initMapSizeAndModelId',
  async ({ queryData }, { getState }): Promise<InitMapSizeAndModelIdActionPayload> => {
    const state = getState();

    return getInitMapSizeAndModelId(state, queryData);
  },
);

export const initMapPosition = createAsyncThunk<
  InitMapPositionActionPayload,
  InitMapPositionParams,
  { dispatch: AppDispatch; state: RootState }
>(
  'map/initMapPosition',
  async ({ queryData }, { getState }): Promise<InitMapPositionActionPayload> => {
    const state = getState();

    return getInitMapPosition(state, queryData);
  },
);

export const initMapBackground = createAsyncThunk<
  InitMapBackgroundActionPayload,
  InitMapBackgroundParams,
  { dispatch: AppDispatch; state: RootState }
>(
  'map/initMapBackground',
  async ({ queryData }, { getState }): Promise<InitMapBackgroundActionPayload> => {
    const state = getState();
    return getBackgroundId(state, queryData);
  },
);

export const initOpenedMaps = createAsyncThunk<
  InitOpenedMapsActionPayload,
  InitOpenedMapsProps,
  { dispatch: AppDispatch; state: RootState }
>('appInit/initOpenedMaps', async ({ queryData }, { getState }): Promise<OppenedMap[]> => {
  const state = getState();

  return getOpenedMaps(state, queryData);
});
